@@ -61,6 +61,7 @@ typedef struct yy_buffer_state *YY_BUFFER_STATE;
 extern YY_BUFFER_STATE yy_scan_bytes (const char *bytes,int len  );
 extern void yy_delete_buffer (YY_BUFFER_STATE  b);
 extern int yylex (void);
+extern void yylex_destroy(void);
 
 /*
   Given a detected URI token, get only the URI specified within.
@@ -74,13 +75,17 @@ extern int yylex (void);
 static char *
 get_uri_string (const char *at, int *pos, int *length)
 {
+  if (*length < 4)
+    return NULL;
+
   if (0 != strncasecmp (at + *pos, "url(", 4))
     return NULL;
 
   *pos += 4;
   *length -= 5; /* url() */
+
   /* skip leading space */
-  while (isspace (at[*pos]))
+  while (*length > 0 && isspace (at[*pos]))
     {
       (*pos)++;
       if (--(*length) == 0)
@@ -88,17 +93,21 @@ get_uri_string (const char *at, int *pos, int *length)
     }
 
   /* skip trailing space */
-  while (isspace (at[*pos + *length - 1]))
+  while (*length > 0 && isspace (at[*pos + *length - 1]))
     {
       (*length)--;
     }
+
   /* trim off quotes */
-  if (at[*pos] == '\'' || at[*pos] == '"')
+  if (*length >= 2 && (at[*pos] == '\'' || at[*pos] == '"'))
     {
       (*pos)++;
       *length -= 2;
     }
 
+  if (*length <= 0)
+    return NULL;
+
   return xstrndup (at + *pos, *length);
 }
 
@@ -139,7 +148,7 @@ get_urls_css (struct map_context *ctx, int offset, int buf_length)
                 {
                   uri = get_uri_string (ctx->text, &pos, &length);
                 }
-              else
+              else if (length >= 2)
                 {
                   /* cut out quote characters */
                   pos++;
@@ -148,6 +157,8 @@ get_urls_css (struct map_context *ctx, int offset, int buf_length)
                   memcpy (uri, yytext + 1, length);
                   uri[length] = '\0';
                 }
+              else
+                uri = NULL;
 
               if (uri)
                 {
@@ -192,6 +203,7 @@ get_urls_css (struct map_context *ctx, int offset, int buf_length)
     }
 
   yy_delete_buffer(b);
+  yylex_destroy();
 
   DEBUGP (("\n"));
 }
